@@ -1,7 +1,7 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 
 """
-Copyright (c) 2006-2019 sqlmap developers (http://sqlmap.org/)
+Copyright (c) 2006-2020 sqlmap developers (http://sqlmap.org/)
 See the file 'LICENSE' for copying permission
 """
 
@@ -10,7 +10,13 @@ from plugins.generic.enumeration import Enumeration as GenericEnumeration
 
 class Enumeration(GenericEnumeration):
     def getPasswordHashes(self):
-        warnMsg = "on DB2 it is not possible to list password hashes"
+        warnMsg = "on IBM DB2 it is not possible to enumerate password hashes"
         logger.warn(warnMsg)
 
         return {}
+
+    def getStatements(self):
+        warnMsg = "on IBM DB2 it is not possible to enumerate the SQL statements"
+        logger.warn(warnMsg)
+
+        return []
